/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: discdomain.h
**  
** Description:
** The header file for a discretized domain. Not a discrete domain. 
**  
** Last modified: 
** 09 January 2007
** 
** Comments:
** 1) [09 Jan 2007] Added the code to sample for each partition
****************************************************************************/

#ifndef DISCDOMAIN_H_
#define DISCDOMAIN_H_

/* For debugging. Comment out to suppress debugging lines */
#define DISCDOMAIN_DEBUG

/* Standard Includes */
#include <iostream>

using namespace std;

namespace domain {

class DiscDomain {

public:

	/* Constructors and destructors */
	DiscDomain(double lowerBound = 0.0, double upperBound = 1.0);
	DiscDomain(const DiscDomain& src);	
	DiscDomain& operator=(const DiscDomain& src);
	virtual ~DiscDomain(); 

	int indexOf(double value);
	int partitionSize();
	bool repartition(int partitions);
	double midValue(int partition);
	double random();
	double randomUnit();
	double random(int index);
	
	/* Printing and debugging */
	string toString() const;
	
protected:
	
	/* Attributes */
	double mLowerBound;
	double mUpperBound;
	int mPartitions;
	double* mUpperBoundaryList;
	
private:
	void copyFrom(const DiscDomain& src);
	
};

}

#endif /* DISCDOMAIN_H_ */
