/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: discdomain_test.cpp
**  
** Description:
** The test file for all the discretized domain
**  
** Last modified: 
** 06 December 2006
** 
** Comments:
**
****************************************************************************/

/* Standard Includes */
#include <iostream>
#include <vector>
#include <map>

/* Application Includes */
#include "discdomain_test.h"
#include "discdomain.h"
#include "random.h"

using namespace domain;
using namespace random;

/** Main function to call for the test of discretize domain
**/
void testDiscDomain() {

	cout << "===================" << endl;
	cout << "In testDiscDomain()" << endl;
	cout << "===================" << endl;

	// Seed the random number generator
	MersenneTwister::init(time(NULL));
	
	cout << "Test 1 : Creation of disc domain" << endl;
	DiscDomain testDomain(1, 10);
	cout << testDomain.toString() << endl;
	cout << endl;
	
	cout << "Test 2 : Repartition" << endl;
	testDomain.repartition(9);
	cout << testDomain.toString() << endl;
	cout << endl;
	
	cout << "Test 3 : Randomly generate number " << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << testDomain.random() << endl;
	cout << endl;
	
	cout << "Test 4 : Test partition bin " << endl;
	cout << "(9) : " << testDomain.indexOf(9.0) << endl;
	cout << "(-1.0) : " << testDomain.indexOf(-1.0) << endl;
	cout << "(90.0) : " << testDomain.indexOf(90.0) << endl;
	cout << "(5.43) : " << testDomain.indexOf(5.43) << endl;
	cout << endl;
	
	cout << "Test 5 : Mid value " << endl;
	cout << "Partition 4 : " << testDomain.midValue(4) << endl;
	cout << "Partition 1 : " << testDomain.midValue(1) << endl;
	cout << "Partition 9 : " << testDomain.midValue(9) << endl;
	cout << endl;
	
	cout << "Test 6 : Number distributions" << endl;
	double histogram[9];
	double value;
	for (int i = 0; i < 9; i++) {
		histogram[i] = 0.0;
	}
	for (int i = 0; i < 100000; i++) {
		value = testDomain.random();
		histogram[testDomain.indexOf(value)]++;
	}
	for (int i = 0; i < 9; i++) {
		histogram[i] = histogram[i] / 100000;
		cout << "Partition [" << i << "] : " << histogram[i] << endl;
	}
	cout << endl;
	
	cout << "End of test" << endl;
}

