/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: factorgraph_test.cpp
**  
** Description:
** The test file for all the factor graph related methods.
**  
** Last modified: 
** 04 December 2006
** 
** Comments:
**
****************************************************************************/

/* Standard Includes */
#include <iostream>
#include <vector>
#include <map>

/* Application Includes */
#include "factorgraph_test.h"
#include "factorgraph.h"

using namespace std;
using namespace factorgraph;

/** Main function to call for the test of the factor graph class.
* It will call the rest of the test, such as basic test, integration tests etc.
**/
void testFactorGraph() {

	testFactorGraphBasic();
	testFactorGraphNodeBasic();
	// testOther();
}

void testFactorGraphBasic() {

	cout << "========================" << endl;
	cout << "In testFactorGraphBasic()" << endl;
	cout << "========================" << endl;

	FactorGraph* graph;
	
	cout << endl;
	cout << "Test 1: Creation of empty factor graph" << endl;
	graph = new FactorGraph();
	cout << graph->toString() << endl;
		
	cout << endl;
	cout << "Test 2: Add in random node" << endl;
	graph->addNode();
	cout << graph->toString() << endl;
	
	cout << endl;
	cout << "Test 3: Add in named node" << endl;
	graph->addNode("x1", "x1", nodeType_factor);
	cout << graph->toString() << endl;
	
	cout << endl;
	cout << "Test 4: Add in node" << endl;
	FactorGraphNode* node = new FactorGraphNode("x2", "x2", nodeType_variable);
	graph->addNode(node);
	cout << graph->toString() << endl;
	
	cout << endl;
	cout << "Test 5: Connecting" << endl;
	graph->connect("x1", "x2");
	cout << graph->toString() << endl;
	
	cout << endl;
	cout << "Test 6: Add in more nodes" << endl;
	graph->addNode(nodeType_factor);
	graph->addNode(nodeType_factor);
	graph->addNode(nodeType_factor);
	graph->addNode(nodeType_variable);
	graph->addNode(nodeType_variable);
	graph->addNode(nodeType_variable);
	cout << graph->toString() << endl;
	
	cout << endl;
	cout << "Test 7: Connects and disconnects and deletes nodes" << endl;
	cout << "Connecting..." << endl;
	graph->connect("f_2", "v_2");
	graph->connect("f_2", "v_1");
	graph->connect("f_2", "v_3");	
	cout << graph->toString() << endl;
	
	cout << "Get neighbors..." << endl;
	vector<FactorGraphNode*> neighborList = graph->getNeighbors("f_2");
	for (int i = 0; i < neighborList.size(); i++) {
		cout << neighborList.at(i)->getId() << " ";
	}
	cout << endl;
	
	cout << "Disconnects..." << endl;
	graph->disconnect("f_2", "v_1");
	cout << graph->toString() << endl;
	
	cout << "Deleting node..." << endl;
	cout << graph->deleteNode("f_2");
	cout << graph->toString() << endl;
			
	delete graph;
	
}

void testFactorGraphNodeBasic() {

	cout << "========================" << endl;
	cout << "In testFactorGraphNodeBasic()" << endl;
	cout << "========================" << endl;

	FactorGraphNode* node;
	
	cout << endl;
	cout << "Test 1: Creation of empty factor node" << endl;
	node = new FactorGraphNode();
	cout << node->toString() << endl;
	delete node;
	
	cout << endl;
	cout << "Test 2: Creation of factor node with default data" << endl;
	node = new FactorGraphNode("testId", "testLabel", nodeType_variable);
	cout << node->toString() << endl;
	delete node;
	
}

void testOther() {

	cout << "==============" << endl;
	cout << "In testOther()" << endl;
	cout << "==============" << endl;
	
}

