/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: maxproductpropagator.h
**  
** Description:
** The header file storing all the specification for a class that performs a max product belief propagation. 
**  
** Last modified: 
** 26 November 2006
** 
** Comments:
****************************************************************************/

#ifndef MAXPRODUCTPROPAGATOR_H_
#define MAXPRODUCTPROPAGATOR_H_

/* For debugging. Comment out to suppress debugging lines */
#define MAXPRODUCTPROPAGATOR_DEBUG

/* Constants */
#define MAXPRODUCTPROPAGATOR_TOLERANCE 0.000001
#define MAXPRODUCTPROPAGATOR_MAXITERATIONS 50

/* Standard Includes */
#include <iostream>
#include <vector>
#include <map>
#include <queue>

/* Application Includes */
#include "factorgraph.h"
#include "discretedistribution.h"

using namespace std;
using namespace factorgraph;
using namespace distribution;

namespace propagator {

class MaxProductPropagator {

public:

	/* Constructors and destructors */
	MaxProductPropagator();
	MaxProductPropagator(const MaxProductPropagator& src);	
	MaxProductPropagator& operator=(const MaxProductPropagator& src);
	virtual ~MaxProductPropagator(); 

	/* Main methods to set up the propagator */
	bool loadFactorGraph(const FactorGraph* graph);
	void unloadFactorGraph();
	bool setDistribution(string id, DiscreteDistribution* distribution, bool isFixed = false);
	DiscreteDistribution* getDistribution(string id);
	bool isFixed(string id);
	double setTolerance(double tolerance);
	double getTolerance();
	int setMaxIterations(int iterations);
	int getMaxIterations();
	
	/* Method to start the propagation */
	bool propagate();
	
protected:

	const FactorGraph* mFactorGraph;
	map<string, DiscreteDistribution*> mBelief;								// Messages here will be DESTROYED AFTER THE WHOLE THING
	map<string, bool> mFixedList;											// Map to whether or not a variable is to be updated
	map<string, queue<DiscreteDistribution*> > mBeliefQueue;				// Messages here will not be destroyed
	map<std::pair<string, string>, DiscreteDistribution*> mMessageHistory;	// Messages here will BE DESTROYED
	
	/* Inference specific attributes */
	double mTolerance;
	int mMaxIterations;
	
	void copyFrom(const MaxProductPropagator& src);
	
};

}

#endif
