/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: matrix_test.cpp
**  
** Description:
** The test file for all the matrix related methods.
**  
** Last modified: 
** 12 February 2007
** 
** Comments:
**
****************************************************************************/

/* Standard Includes */
#include <iostream>
#include <math.h>

/* Application Includes */
#include "matrix_test.h"
#include "matrix.h"

using namespace matrix;

/** Main function to call for the test of the matrices
* It will call the rest of the tests.
**/
void testMatrix() {

	// testMatrixSimple();
	testluDecompose();
	testInverse2();
}

void testMatrixSimple() {

	cout << "=====================" << endl;
	cout << "In testMatrixSimple()" << endl;
	cout << "=====================" << endl;

	Matrix mat1(2, 2);
	Matrix mat2(2, 2);
	
	mat1(0, 0) = 1;	mat1(0, 1) = 2;
	mat1(1, 0) = 3;	mat1(1, 1) = 4;
	
	mat2(0, 0) = 1;	mat2(0, 1) = 2;
	mat2(1, 0) = 3;	mat2(1, 1) = 4;
	
	Matrix* mat3;
	mat3 = mat1 * mat2;
	Matrix* mat4;
	mat4 = transpose(*mat3);

	cout << mat3->toString(4) << endl;
	cout << mat4->toString(4) << endl;
	delete mat3;
	delete mat4;
}

void testluDecompose() {

	cout << "=====================" << endl;
	cout << "In testluDecompose()" << endl;
	cout << "=====================" << endl;

	Matrix mat(4, 4);
	mat(0, 0) = 10;	mat(0, 1) = 2;	mat(0, 2) = 3;	mat(0, 3) = 4;
	mat(1, 0) = 5;	mat(1, 1) = 6;	mat(1, 2) = 7;	mat(1, 3) = 8;
	mat(2, 0) = 9;	mat(2, 1) = 10;	mat(2, 2) = 11;	mat(2, 3) = 12;
	mat(3, 0) = 13;	mat(3, 1) = 14;	mat(3, 2) = 15;	mat(3, 3) = 1;
		
	double d;
	vector<int> indexArray;
	luDecompose(mat, indexArray, d);
	
	cout << mat.toString(3) << endl;
	cout << "Permutation matrix" << endl;
	for (int i = 0; i < indexArray.size(); i++) cout << indexArray[i] << " ";
	cout << endl;
	
}

void testInverse() {

	cout << "=====================" << endl;
	cout << "In testInverse()" << endl;
	cout << "=====================" << endl;

	Matrix mat(4, 4);
	
	mat(0, 0) = 10;	mat(0, 1) = 2;	mat(0, 2) = 3;	mat(0, 3) = 4;
	mat(1, 0) = 5;	mat(1, 1) = 6;	mat(1, 2) = 7;	mat(1, 3) = 8;
	mat(2, 0) = 9;	mat(2, 1) = 10;	mat(2, 2) = 11;	mat(2, 3) = 12;
	mat(3, 0) = 13;	mat(3, 1) = 14;	mat(3, 2) = 15;	mat(3, 3) = 1;

	cout << "Computing the inverse" << endl;
	inverse(mat);
	cout << mat.toString(3) << endl;

}

void testInverse2() {

	cout << "=====================" << endl;
	cout << "In testInverse2()" << endl;
	cout << "=====================" << endl;

	Matrix mat(5, 5);
	mat(0, 0) = 6;	mat(0, 1) = 2;	mat(0, 2) = 8;	mat(0, 3) = 6; mat(0, 4) = 5;
	mat(1, 0) = 4;	mat(1, 1) = 9;	mat(1, 2) = 8;	mat(1, 3) = 7; mat(1, 4) = 2;
	mat(2, 0) = 9;	mat(2, 1) = 7;	mat(2, 2) = 5;	mat(2, 3) = 3; mat(2, 4) = 0;
	mat(3, 0) = 1;	mat(3, 1) = 4;	mat(3, 2) = 1;	mat(3, 3) = 2; mat(3, 4) = 10;
	mat(4, 0) = 4;	mat(4, 1) = 1;	mat(4, 2) = 0;	mat(4, 3) = 0; mat(4, 4) = 4;
	
	cout << "Computing the inverse" << endl;
	inverse(mat);
	cout << mat.toString(3) << endl;
}
