/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: odecommon.h
**  
** Description:
** This is the implementation of the common functionalities that will be used in the ode package
**
** Last modified: 
** 06 April 2007
** 
** Comments:
**
****************************************************************************/

#ifndef ODECOMMON_H_
#define ODECOMMON_H_

/* For debugging. Comment out to suppress debugging lines */
#define ODECOMMON_DEBUG

/* Standard Includes */
#include <iostream>
#include <vector>

/* Application Includes */
#include "odefunction.h"

using namespace std;

namespace ode {

typedef enum {

	// The types for the basic operators
	nodeType_operator_plus,
	nodeType_operator_minus,
	nodeType_operator_times, 
	nodeType_operator_divide,
	nodeType_operator_power,
	
	// The types for the functions
	nodeType_function_abs, 
	nodeType_function_arccos, 
	nodeType_function_arccosh, 
	nodeType_function_arccot, 
	nodeType_function_arccoth, 
	nodeType_function_arccsc,
	nodeType_function_arccsch, 
	nodeType_function_arcsec, 
	nodeType_function_arcsech, 
	nodeType_function_arcsin, 
	nodeType_function_arcsinh,
	nodeType_function_arctan, 
	nodeType_function_arctanh, 
	nodeType_function_ceiling, 
	nodeType_function_cos, 
	nodeType_function_cosh, 
	nodeType_function_cot,
	nodeType_function_coth, 
	nodeType_function_csc, 
	nodeType_function_csch, 
	nodeType_function_exp, 
	nodeType_function_factorial, 
	nodeType_function_floor, 
	nodeType_function_ln,
	nodeType_function_log, 
	nodeType_function_sec, 
	nodeType_function_sech, 
	nodeType_function_sin, 
	nodeType_function_sinh, 
	nodeType_function_sqrt, 
	nodeType_function_tan, 
	nodeType_function_tanh,	

	// The types for symbols and values
	nodeType_symbol,
	nodeType_symbol_parameter,
	nodeType_symbol_dvariable,
	nodeType_symbol_ivariable,
	nodeType_value,
	
	// Unknown type
	nodeType_unknown
	
} ODENodeType;

bool isValidSymbol(string token);
bool isNumeric(string token);
double toDouble(string token);
bool isOperator(string token);
bool isFunction(string token);
int getPriority(string token);
ODENodeType getType(string token);
ODEFunction* createFunction(string token);
ODEFunction* createFunction(ODENodeType type);
string typeToString(ODENodeType type);
bool multiplyVector(vector<double>& result, double h, const vector<double>& y);
bool addVector(vector<double>& result, const vector<double>& y1, const vector<double>& y2);
bool subtractVector(vector<double>& result, const vector<double>& y1, const vector<double>& y2);
double magnitude(const vector<double>& v);

}

#endif /* ODECOMMON_H_ */
