/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: odenode.h
**  
** Description:
** This is the implementation of a node in the ODE system. A node is a single token/element in the
** equation that describes the system.
**
** Last modified: 
** 20 February 2007
** 
** Comments:
**
****************************************************************************/

#ifndef ODENODE_H_
#define ODENODE_H_

/* For debugging. Comment out to suppress debugging lines */
#define ODENODE_DEBUG

/* Standard Includes */
#include <iostream>
#include <vector>

/* Application Includes */
#include "odecommon.h"
#include "odesystem.h"
#include "odeparser.h"
#include "odefunction.h"

using namespace std;

namespace ode {

/* Forward Declarations */
class ODESystem;

class ODENode {

public:

	/* Constructors and destructors */
	ODENode(ODENodeType type, ODENode* parent = 0, ODESystem* context = 0);
	ODENode(string token, ODENode* parent = 0, ODESystem* context = 0);
	ODENode(const ODENode& src);
	ODENode& operator=(const ODENode& src);
	virtual ~ODENode(); 

	/* Gets the type of the symbol */
	ODENodeType getType(string token);

	/* Sets the value and symbol */
	bool setSymbol(string symbol);
	bool setValue(double value);
	
	/* Adding children */
	ODENode* addChild(ODENode* child);
	ODENode* removeChild(ODENode* child);
	
	/* Establishing context links */
	bool setContext(ODESystem* context);
	bool unsetContext();
	
	/* Evaluating the node */
	double evaluate();
	double evaluate(const vector<double>& y, const vector<double>& u, const vector<double>& p);
	
	/* Printing and debugging */
	string toStringSimple() const;
	string toString() const;

protected:

	ODENodeType mType;								
	string mSymbol;							// These member attributes are only to be used if the node
	int mIndex;								// is a symbol or an actual value 
	double mValue;
	
	ODENode* mParent;						// The immediate parent of the node
	ODESystem* mContext;					// The context (system) of the node
	vector<ODENode*> mChildList;			// The child list
	ODEFunction* mFunction;					// The function of the system

private:
	void copyFrom(const ODENode& src);
	
};

}

#endif /* ODENODE_H_ */
