/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: odetokenizer.h
**  
** Description:
** This is the implementation of an equation tokenizer for an ODE.
**
** Last modified: 
** 23 February 2007
** 
** Comments:
**
****************************************************************************/

#ifndef ODETOKENIZER_H_
#define ODETOKENIZER_H_

/* For debugging. Comment out to suppress debugging lines */
#define ODETOKENIZER_DEBUG

/* Standard Includes */
#include <iostream>
#include <vector>

using namespace std;

namespace ode {

class ODETokenizer {

public:

	/* Constructors and destructors */
	ODETokenizer();
	ODETokenizer(string expression);
	ODETokenizer(const ODETokenizer& src);	
	ODETokenizer& operator=(const ODETokenizer& src);
	virtual ~ODETokenizer(); 

	/* Functions to get next token */
	string nextToken();
	string peek();
	bool hasMoreTokens();
	
	/* Tokenize the string  */
	bool tokenize(string expression);
	
	/* Printing and debugging */
	string toString() const;
	
protected:
	vector<string> mBuffer;

private:
	void copyFrom(const ODETokenizer& src);
	
};

}

#endif /* ODETOKENIZER_H_ */
