/****************************************************************************
** Copyright (C) 2006 Koh Yeow Nam, Geoffrey
** All rights reserved
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Filename: odetrace.h
**  
** Description:
** This is a wrapper for a 2D array representing the trace results from the simulations
**
** Last modified: 
** 26 March 2007
** 
** Comments:
** (1) The empty list member variable is used to signal errors in locating the trace.
****************************************************************************/

#ifndef ODETRACE_H_
#define ODETRACE_H_

/* For debugging. Comment out to suppress debugging lines */
#define ODETRACE_DEBUG

/* Standard Includes */
#include <iostream>
#include <vector>

using namespace std;

namespace ode {

class ODETrace {
public:
	/* Constructors and destructors */
	ODETrace();
	ODETrace(const ODETrace& src);	
	ODETrace& operator=(const ODETrace& src);
	virtual ~ODETrace(); 

	/* Functions to add variables */
	bool addVariable(string variable);
	bool addVariableList(const vector<string>& variableNameList);
	
	/** Accessors and mutator functions to add and retrienve traces **/
	const vector<double>& getTimeList() const;
	const vector<double>& getTrace(string variable) const;
	const vector<double>& getTrace(int index) const;
	bool addTraceAt(double time, const vector<double>& trace);
	
	/** Additional functions to add in more complicated time looking mechanisms including simple linear 
	       interpolation (* To be implemented later *) **/
	// double getTraceAt(double time, string variable);
	// double getTraceAt(double time, int index);
	// vector<double>& getTraceAt(double time);
	double getValueAt(string variable, int timeIndex) const;
	double getValueAt(int varIndex, int timeIndex) const;
	void setValueAt(string variable, int timeIndex, double value);
	void setValueAt(int varIndex, int timeIndex, double value);
	
	/** Functions to write to output **/
	bool save(const string filename, string delimiter = "\t") const; 
	bool load(const string filename, string delimiter = "\t");
	
	int indexOf(string variable) const;
	int timeIndexOf(double time) const;
	int stepSize() const;
	int variableSize() const;
	
protected:
	vector<double> mTimeList;
	vector< vector<double> > mTraceList;
	vector<string> mVariableNameList;
	const vector<double> mEmptyList;
	bool mInitialized;

private:
	void copyFrom(const ODETrace& src);

};

}

#endif /* ODETRACE_H_ */
